#ifndef N_WATERNODE_H
#define N_WATERNODE_H
//------------------------------------------------------------------------------
/**
    @class nWaterNode

    @brief a brief description of the class

    a detailed description of the class

    (C)	2002	tom@3d-inferno.com
*/
#ifndef N_VISNODE_H
#include "node/nvisnode.h"
#endif

#ifndef N_REF_H
#include "kernel/nref.h"
#endif

#ifndef N_DYNVERTEXBUFFER_H
#include "gfx/ndynvertexbuffer.h"
#endif

#ifndef N_DYNAUTOREF_H
#include "kernel/ndynautoref.h"
#endif

#ifndef N_KERNELSERVER_H
#include "kernel/nkernelserver.h"
#endif

#ifndef N_WATERSERVER_H
#include "water/nwaterserver.h"
#endif

#undef N_DEFINES
#define N_DEFINES nWaterNode
#include "kernel/ndefdllclass.h"


//------------------------------------------------------------------------------
class nGfxServer;
class nIndexBuffer;

//------------------------------------------------------------------------------
//helper function to compute the power of 2 of given x
inline int PowerOf2(int pow)
{
	int erg=0;
	while(pow > 1)
	{
		erg++;
		pow=pow>>1;
	}
	return pow<<erg;
}

class N_DLLCLASS nWaterNode : public nVisNode
{
private:
	int m_Size;
	int m_NSize; //new x
	int m_Type;
	int m_Nt;
	bool m_Render;
	nRef <nVertexBuffer> ref_vb;
	nAutoRef<nWaterServer> ref_Server;
	nRef<nIndexBuffer> ref_ibuf;
	bool initIndex();
	bool initBuffer();
protected:
	int localChannelIndex;
public:
	nWaterNode();
	~nWaterNode();
	virtual bool SaveCmds(nFileServer* fileServer);
    /// attach to scene
    virtual bool Attach(nSceneGraph2*);
    /// update internal state and render
    virtual void Compute(nSceneGraph2*);
	void AttachChannels(nChannelSet* chnSet);
	/// set as shadow caster state
    inline void SetCastShadow(bool b){};
    /// can cast shadows?
    bool GetCastShadow(){return false;};
    /// set read only flag
    void SetReadOnly(bool b){};
    /// get readonly flag
    bool GetReadOnly(){return false;};
	bool  SetType(int _t);
	int GetType();
	inline int GetSize(){ return this->m_Size;};
	float GetHeight(float i, float j);
	void SetSize(int i);

    static nClass* local_cl;
    static nKernelServer* kernelServer;
};

inline nWaterNode::nWaterNode():
	ref_Server(kernelServer, this),
    ref_ibuf(this),
	ref_vb(this),
	m_Size(SampleSize),//SampleSize),
	m_Type(0),
	m_NSize(SampleSize),
	m_Nt(0),
	m_Render(false)
{
    this->ref_Server = "/sys/servers/water";
	nWaterServer * S = this->ref_Server.get();
	S->AddWaterRef((int)0);
	m_Size=PowerOf2(SampleSize);
}

inline nWaterNode::~nWaterNode()
{
	if(this->ref_Server.isvalid())
	{
		nWaterServer * S = this->ref_Server.get();
		S->RemWaterRef(this->m_Type);
		this->ref_Server.invalidate();
	}

    // release vertex buffer
    if (this->ref_vb.isvalid()) {
        this->ref_vb->Release();
        this->ref_vb.invalidate();
    }

    // release index buffer
    if (this->ref_ibuf.isvalid()) {
        this->ref_ibuf->Release();
        this->ref_ibuf.invalidate();
    }
}

inline float nWaterNode::GetHeight(float i, float j)
{
	if(ref_Server.isvalid())
	{
		nWaterServer * S = this->ref_Server.get();
		return S->GetHeight(i,j,this->m_Type);

	}
	else
		return 0.0f;
}

//0 Stat , 1 Chop, 2 Gerstner
inline bool nWaterNode::SetType(int i)
{
	if(i < 0 || i> 3)
		return false;
	else
		this->m_Nt = i;
	return true;
}

inline int nWaterNode::GetType()
{
    return this->m_Type;
}
//------------------------------------------------------------------------------
/**
    Set size of water field must be power of 2

	@param	ZSize
*/
inline void nWaterNode::SetSize(int i)
{
	n_assert(i > 0)

	this->m_NSize = PowerOf2(i);
}
#endif

