#ifndef N_WATERSERVER_H
#define N_WATERSERVER_H
//--------------------------------------------------------------------
/**
    @class nWaterServer

    @brief Nebula console server

    The water server can be found at "/sys/servers/water".
*/
//--------------------------------------------------------------------
#ifndef N_TYPES_H
#include "kernel/ntypes.h"
#endif

#ifndef N_ROOT_H
#include "kernel/nroot.h"
#endif

#ifndef N_AUTOREF_H
#include "kernel/nautoref.h"
#endif

#ifndef N_DYNVERTEXBUFFER_H
#include "gfx/ndynvertexbuffer.h"
#endif

//#include "water/Types.h"
#include "water/choppywaves.h"
#include "water/gerstnerwaves.h"

//--------------------------------------------------------------------
#undef N_DEFINES
#define N_DEFINES nWaterServer
#include "kernel/ndefdllclass.h"
//--------------------------------------------------------------------
class nGfxServer;
class nWaterNode;
class GerstnerWaves;
class ChoppyWaves;
const int SampleSize = 32;
const int VertType = (N_VT_COORD|N_VT_RGBA|N_VT_UV0|N_VT_NORM);
class N_DLLCLASS nWaterServer : public nRoot {
public:
    static nClass *local_cl;
    static nKernelServer *kernelServer;
	enum N_WATER
	{
		N_STATISTIC = 0,
		N_CHOPPY,
		N_GERSTNER,
		N_CHOPPY_3TO1
	};

	nWaterServer();
	~nWaterServer();
	bool AddWaterRef(int);
	bool RemWaterRef(int);
	void SetFPS(int);
	int GetFPS();
	float GetHeight(int x, int z, int algo);
	float GetHeight(float x, float z, int algo);
	bool Update(float _t);

	const nComplex *getCMap(){return m_Choppy->getMap();};
	const nComplex *getXMap(){return m_Choppy->getDeltaMapX();};
	const nComplex *getZMap(){return m_Choppy->getDeltaMapZ();};
	vector3 *getGMap(){return m_Gerstner->getVertices();};
	int GetRefs(){return m_A + m_B + m_C;};
	float GetChopp(){ return (m_A||m_B) ? m_Choppy->getLambda() : 0.0f;	};
	void SetChopp(float _t){ if(m_A||m_B) m_Choppy->setLambda(_t);};
	void Create3to1Mesh(bool f){m_3to1Mesh =f;};
private:
	void updateChoppy(void);// choppy waves update also updates Statistic waves
	void updateStatistc(void);//use if only Statistic waves present
	void updateGerstner(void);//gerstner waves
//	void UpdateCNormals(void);//create a normal array for choppy waves 
//	void UpdateCDirect(void);//write directly to buffers without filling arrays for fast cpu's
//	void UpdateSNormals(void);
	void update3to1(int);
	void initIndex(void);
	void init3to1Index(void);
	void test();
	float GetSH(float x, float z);
	nRef<nVertexBuffer> c_vb;
	nRef<nVertexBuffer> c_3to1vb;
	nRef<nVertexBuffer> s_vb;
	nRef<nVertexBuffer> s_3to1vb;
	nRef<nVertexBuffer> g_vb;
	nRef<nVertexBuffer> g_3to1vb;
	nRef<nIndexBuffer> ref_ibuf;
	nRef<nIndexBuffer> ref_i3to1buf;
	ChoppyWaves *m_Choppy;
	GerstnerWaves *m_Gerstner;
	nAutoRef<nGfxServer> refGfx;
	bool m_3to1Mesh;
	int m_A;
	int m_B;
	int m_C;
	int m_FPS;
	float m_delta;
	float m_last;
	int m_Updated;
};



//--------------------------------------------------------------------
#endif