#define N_IMPLEMENTS nWaterServer

//------------------------------------------------------------------------------
//  (C) 2002	tom@3d-inferno.com
//------------------------------------------------------------------------------
#include "gfx/nvertexbuffer.h"
#include "gfx/ngfxserver.h"
#include "gfx/nscenegraph2.h"
#include "gfx/nindexbuffer.h"
#include "math/nmath.h"
#include "kernel/ntypes.h"
#include "water/nwaterserver.h"

nWaterServer::nWaterServer()
:m_Choppy(NULL), 
m_Gerstner(NULL), 
m_3to1Mesh(false), 
m_A(0), 
m_B(0), 
m_C(0), 
m_Updated(0),
c_vb(NULL),
s_vb(NULL),
g_vb(NULL),
refGfx(kernelServer, this)
{
	this->refGfx     = "/sys/servers/gfx";
	this->SetFPS(24);
}

nWaterServer::~nWaterServer()
{
	if(this->m_Choppy)
	{
		n_delete this->m_Choppy;
		this->m_Choppy = NULL;
	}
	if(this->m_Gerstner)
	{
		n_delete this->m_Gerstner;
		this->m_Gerstner = NULL;
	}
};


bool nWaterServer::AddWaterRef(int _ref)
{
	switch(_ref)
	{
	case N_STATISTIC:
		if((!m_Choppy) && m_A ==0 )
		{
			m_Choppy = n_new ChoppyWaves(-0.15f, SampleSize, SampleSize,
				0.0008f, vector2(30,10), 100.0f, 100.0f);
		}
		m_A++; 
		return true;
	case N_CHOPPY:
		if((!m_Choppy) && m_B ==0 )
		{
			m_Choppy = n_new ChoppyWaves(-0.15f, SampleSize, SampleSize, 
				0.0008f, vector2(30,10), 100.0f, 100.0f);
		}
		m_B++; return true;
	case N_GERSTNER:
		if((!m_Gerstner) && m_C ==0 )
		{
			m_Gerstner = n_new GerstnerWaves(SampleSize, SampleSize);
			m_Gerstner->addWave(vector2(1, 1), 5, 0.1f, 1);
			m_Gerstner->addWave(vector2(2, 1), 20, 0.8f, 3);
			m_Gerstner->addWave(vector2(3, 2), 10, 0.3f, 2);
			m_Gerstner->addWave(vector2(2, 1), 40, 1.4f, 9);
		}
		m_C++; return true;
//	case N_CHOPPY_3TO1:
//		m_3to1Mesh=true;return true;
	default: return false;
	}
}

bool nWaterServer::RemWaterRef(int _ref)
{
	switch(_ref)
	{
	case N_STATISTIC:
		m_A--;
		if(!m_A)
		{
			if(!m_B)
			{
				n_delete m_Choppy;
				m_Choppy = NULL;
			}
		}
		n_assert(m_A>-1);return true;
	case N_CHOPPY:
		m_B--;
		if(!m_A)
		{
			if(!m_B)
			{
				n_delete m_Choppy;
				m_Choppy = NULL;
			}
		}
		n_assert(m_B>-1);return true;
	case N_GERSTNER:
		m_C--;
		if(!m_C)
		{
			n_delete m_Gerstner;
			m_Gerstner = NULL;
		}
		n_assert(m_C>-1);return true;
	default: return false;
	}
}

int nWaterServer::GetFPS()
{
	return this->m_FPS;
}

void nWaterServer::SetFPS(int i)
{
	n_assert(i > 0)
    this->m_FPS = i;
	this->m_delta = (float)1/i;
}

bool nWaterServer::Update(float _t)
{
	if((_t - this->m_last) > this->m_delta)
	{
		//save updated time
		this->m_last=_t;
		//check if index is invalid
		if(!this->ref_ibuf.isvalid())
			initIndex();

		if(m_Choppy && !m_Updated)
			m_Choppy->update(_t);
		//update each working algorithm in a different time frame for balance
		switch(m_Updated)
		{
		case 0:
			if(m_A)
				this->updateStatistc();
//			if(this->m_3to1Mesh)//test only
//				this->update3to1(1);
			m_Updated++;
			break;
		case 1:
			if(m_B)
			{
				this->updateChoppy();
//				if(this->m_3to1Mesh)
//				this->update3to1(1);
			}
				m_Updated++;
			break;
		case 2:
			if(m_C && m_Gerstner)
			{
				m_Gerstner->update(_t);
				this->updateGerstner();
			}
			m_Updated=0;
		default:
			m_Updated=0;
		}
		return true;
	}
	else
		return false;
}

void nWaterServer::updateChoppy()
{
	vector3 v1,vbase,no;
	const nComplex * xmap = getXMap();
	const nComplex * ymap = getCMap();
	const nComplex * zmap = getZMap();

	//check if buffer is valid
	if(!this->c_vb.isvalid())
	{
		char iname[256];
		//that's how waternodes will find the buffer so don't change the name
		//if you don't change it in nwaternode.cc!
		sprintf(iname,"nWaterNode_%d_%d",N_CHOPPY,SampleSize);
		//first try to fid it then create a new !
		this->c_vb = this->refGfx->FindVertexBuffer(iname);
		if(this->c_vb.isvalid())
		{
			((nVertexBuffer *)(this->c_vb.get()))->AddRef();
		}
		else
		{	//just a write only buffer as noone wants to read it just attach, attach...
			this->c_vb=this->refGfx->NewVertexBuffer(iname,N_VBTYPE_STATIC,VertType,(SampleSize	+1)*(SampleSize	+1));
		}
	}
	//lock it for overwriting
	this->c_vb->LockVertices();
    // get the required pointers
    float *coord = this->c_vb->coord_ptr;
	float *norm = this->c_vb->norm_ptr;
    ulong *color = this->c_vb->color_ptr;
    float *uv0 = this->c_vb->uv_ptr[0];
	ulong ccol = n_f2rgba(1.0f,1.0f,1.0f,1.0f); 

	int tot=0;//total vertices processed
	//remember this stretches a 32 *32 complex field to a 33*33 vertex field
	for(int rows=0, a, b; rows <= SampleSize; rows++)
	{//a and b are used to have tilable sides (same verts at oposite side)
		if(rows>=SampleSize)
			b=0;
		else
			b=rows;
		for(int cols=0; cols <=SampleSize; cols++,tot++)
		{	
			if(cols>=SampleSize)
				a=0;
			else
				a=cols;
		
			if(a==0)//no x deviation
			{
				v1.x=(float)cols;
			}
			else
			{
				v1.x=(float) a + xmap[a + (b * SampleSize)].real()*0.4f;
			}
			
			if(b==0)//||rows==(SampleSize))//no z deviation
			{
				v1.z=(float)rows;
			}
			else //z deviation
			{
				v1.z=(float) b + zmap[a + (b * SampleSize)].real()*0.4f;
			}
			v1.y=ymap[a + (b * SampleSize)].real()*0.04f;

			coord[0] = v1.x;
			coord[1] = v1.y;
			coord[2] = v1.z;

			norm[0] = 0.0f;
			norm[1] = v1.y*v1.y;//*v1.y;
			norm[2] = 0.0f;

			color = &ccol;// colors are all the same this could be chnaged in future

			uv0[0] = ((float)(cols)/(SampleSize+1));
			uv0[1] = ((float)(rows)/(SampleSize+1));

			uv0 += this->c_vb->stride4;
			color += this->c_vb->stride4;
			norm += this->c_vb->stride4;
			coord += this->c_vb->stride4;
		}//end of cols
	}//end of rows
	//relesae lock
	this->c_vb->UnlockVertices();
}

void nWaterServer::updateStatistc()
{
	float height;
	const nComplex * map = getCMap();
	
	if(!this->s_vb.isvalid())
	{
		char iname[256];
		sprintf(iname,"nWaterNode_%d_%d",N_STATISTIC,SampleSize);
		//first try to fid it then create a new !
		this->s_vb = this->refGfx->FindVertexBuffer(iname);
		if(this->s_vb.isvalid())
		{
			((nVertexBuffer *)(this->s_vb.get()))->AddRef();
		}
		else
		{
			this->s_vb = this->refGfx->NewVertexBuffer(iname,N_VBTYPE_STATIC,VertType,(SampleSize	+1)*(SampleSize	+1));
		}
	}
		
	this->s_vb->LockVertices();
    // get the required pointers
    float *coord = this->s_vb->coord_ptr;
	float *norm = this->s_vb->norm_ptr;
    ulong *color = this->s_vb->color_ptr;
    float *uv0 = this->s_vb->uv_ptr[0];
	ulong ccol = n_f2rgba(1.0f,1.0f,1.0f,1.0f); 

	int tot=0;//total vertices processed
	//remember this stretches a 32 *32 complex field to a 33*33 vertex field
	for(int rows=0, a, b; rows <= SampleSize; rows++)
	{//a and b are used to have tilable sides (same verts at oposite side)
		if(rows>=SampleSize)
			b=0;
		else
			b=rows;
		for(int cols=0; cols <=SampleSize; cols++,tot++)
		{	
			if(cols>=SampleSize)
				a=0;
			else
				a=cols;

			height=map[a + (b * SampleSize)].real()*0.04f;
			coord[0] = (float)cols;
			coord[1] = height;
			coord[2] = (float)rows;
			//normal base is always 0 in x and z direction, not normalized like choppy
			norm[0] = 0.0f;
			norm[1] = height*height;//for always positive values
			norm[2] = 0.0f;

			color = &ccol;// colors are all the same this could be chnaged in future

			uv0[0] = ((float)(cols)/(SampleSize+1));
			uv0[1] = ((float)(rows)/(SampleSize+1));

			uv0 += this->s_vb->stride4;
			color += this->s_vb->stride4;
			norm += this->s_vb->stride4;
			coord += this->s_vb->stride4;
		}//end of cols
	}//end of rows
	this->s_vb->UnlockVertices();
}

void nWaterServer::updateGerstner()
{
	vector3 v1,vbase,no;
	const vector3 * map = getGMap();
 
   	if(!this->g_vb.isvalid())
	{
		char iname[256];
		sprintf(iname,"nWaterNode_%d_%d",N_GERSTNER,SampleSize);
		//first try to find it then create a new !
		this->g_vb = this->refGfx->FindVertexBuffer(iname);
		if(this->g_vb.isvalid())
		{
			((nVertexBuffer *)(this->g_vb.get()))->AddRef();
		}
		else
		{
			this->g_vb = this->refGfx->NewVertexBuffer(iname,N_VBTYPE_STATIC,VertType,(SampleSize +1)*(SampleSize +1));
		}
	}
		
	this->g_vb->LockVertices();
	float *coord = this->g_vb->coord_ptr;
	float *norm = this->g_vb->norm_ptr;
    ulong *color = this->g_vb->color_ptr;
    float *uv0 = this->g_vb->uv_ptr[0];
	ulong ccol = n_f2rgba(1.0f,1.0f,1.0f,1.0f); 

	int tot=0;//total vertices processed
	//remember this stretches a 32 *32 complex field to a 33*33 vertex field
	for(int rows=0, a, b; rows <= SampleSize; rows++)
	{//a and b are used to have tilable sides (same verts at oposite side)
		if(rows>=SampleSize)
			b=0;
		else
			b=rows;
		for(int cols=0; cols <=SampleSize; cols++,tot++)
		{	
			if(cols>=SampleSize)
				a=0;
			else
				a=cols;
			
			if(a==0)//no x deviation
			{
				v1.x=(float)cols;
			}
			else
			{
				v1.x=map[a + (b * SampleSize)].x;//(float) a + map[a + (b * SampleSize)].x*0.02f;
//			n_clamp(v1.x, 0.1f,31.9f);
			}
			
			
			if(b==0)//||rows==(SampleSize))//no z deviation
			{
				v1.z=(float)rows;
			}
			else //z deviation
			{
				v1.z=map[a + (b * SampleSize)].z;//(float) b + map[a + (b * SampleSize)].z*0.02f;
//				n_clamp(v1.x, 0.1f,31.9f);
			}
			v1.y=map[a + (b * SampleSize)].y;

			coord[0] = v1.x;
			coord[1] = v1.y;
			coord[2] = v1.z;

			norm[0] = 0.0f;//v1.x - (float)cols;
			norm[1] = v1.y*v1.y;
			norm[2] = 0.0f;//v1.z - (float)rows;

			color = &ccol;// colors are all the same this could be chnaged in future

			uv0[0] = ((float)(cols)/(SampleSize+1));
			uv0[1] = ((float)(rows)/(SampleSize+1));

			uv0 += this->g_vb->stride4;
			color += this->g_vb->stride4;
			norm += this->g_vb->stride4;
			coord += this->g_vb->stride4;
		}//end of cols
	}//end of rows
	this->g_vb->UnlockVertices();
}

void nWaterServer::initIndex(void)
{
    n_assert(!this->ref_ibuf.isvalid());
	char iname[256];
	sprintf(iname,"nWaterNode_%d", SampleSize);
	//find index
	this->ref_ibuf = this->refGfx->FindIndexBuffer(iname);
    if(this->ref_ibuf.isvalid())
	{
        ((nIndexBuffer *)(this->ref_ibuf.get()))->AddRef();
	}
	else
	{
		// create index buffer
		this->ref_ibuf = this->refGfx->NewIndexBuffer(iname);
		n_assert(this->ref_ibuf.isvalid());
		
		// each 4 verts will be 2 tris or 6 indices  
		// 01  and  1
		// 2       23
		nIndexBuffer *ib = this->ref_ibuf.get();

		ib->Begin(N_IBTYPE_STATIC, N_PTYPE_TRIANGLE_LIST, ((SampleSize)*(SampleSize) * 6));
		int i,j;
		int act_v = ((SampleSize+1) * (SampleSize));// ((SampleSize+1) * (SampleSize));//act_v = (actSize)*(actSize - 1);
		for (j=0, i=0 ; i < act_v; i++) 
		{	
			if(((i+1)%(SampleSize+1))==0)
			{
				continue;
			}
			ib->Index(j,i);
			ib->Index(j+1,i+(SampleSize+1));
			ib->Index(j+2,i+1);
			
			j+=3;
			ib->Index(j,i+1);
			ib->Index(j+1,i+(SampleSize+1));
			ib->Index(j+2,i+(SampleSize+1)+1);
			
			j+=3;
		}
		ib->End();
	}
//	return true;
}

void nWaterServer::init3to1Index(void)
{
    n_assert(!this->ref_i3to1buf.isvalid());
	char iname[256];
	sprintf(iname,"nWaterNode_3to1_%d", SampleSize);
n_printf("Server init3to1index \n");
	//find index
	this->ref_i3to1buf = this->refGfx->FindIndexBuffer(iname);
    if(this->ref_i3to1buf.isvalid())
	{
        ((nIndexBuffer *)(this->ref_i3to1buf.get()))->AddRef();
n_printf("initindex found\n");
	}
	else
	{
		// create index buffer
		this->ref_i3to1buf = this->refGfx->NewIndexBuffer(iname);
		n_assert(this->ref_i3to1buf.isvalid());

		nIndexBuffer *ib = this->ref_i3to1buf.get();

		ib->Begin(N_IBTYPE_STATIC, N_PTYPE_TRIANGLE_LIST, ((8*3*4)*(SampleSize-1)));

		for (int a=0, b=(SampleSize*12),i=0; i<(SampleSize*4*12); i+=24,a+=6,b+=2) 
		{	
			ib->Index(i,b);	ib->Index(i+1,a); ib->Index(i+2,a+1);
//n_printf("%d %d %d\n",b,a,a+1);
			ib->Index(i+3,a+1); ib->Index(i+4,a+2); ib->Index(i+5,b);
//n_printf("%d %d %d\n",a+1,a+2,b);
			ib->Index(i+6,b); ib->Index(i+7,a+2); ib->Index(i+8,b+1);
//n_printf("%d %d %d\n",b,a+2,b+1);
			ib->Index(i+9,b+1); ib->Index(i+10,a+2); ib->Index(i+11,a+3);
//n_printf("%d %d %d\n",b+1,a+2,a+3);
			ib->Index(i+12,a+3); ib->Index(i+13,a+4); ib->Index(i+14,b+1);
//n_printf("%d %d %d\n",a+3,a+4,b+1);
			ib->Index(i+15,b+1); ib->Index(i+16,a+4); 
			if(i>=(SampleSize*48)-24)
			{	
				ib->Index(i+17,((SampleSize)*12));
//n_printf("%d %d %d\n",b+1,a+4,((SampleSize)*12));
				ib->Index(i+18,((SampleSize)*12));
				ib->Index(i+19,a+4); ib->Index(i+20,a+5);
//n_printf("%d %d %d\n",((SampleSize)*12),a+4,a+5);
				ib->Index(i+21,a+5); ib->Index(i+22,a+6);
				ib->Index(i+23,0);
//n_printf("%d %d %d     i:%d\n",a+5,a+6,0,i+23);
			}
			else
			{
				ib->Index(i+17,b+2);
//n_printf("%d %d %d\n",b+1,a+4,b+2);
				ib->Index(i+18,b+2);
				ib->Index(i+19,a+4); ib->Index(i+20,a+5);
//n_printf("%d %d %d\n",b+2,a+4,a+5);
				ib->Index(i+21,a+5); ib->Index(i+22,a+6);
				ib->Index(i+23,b+2);
//n_printf("%d %d %d     i:%d\n",a+5,a+6,b+2,i+23);
			}
		}
		ib->End();
n_printf("initindex new \n");
	}
//	return true;
}

void nWaterServer::update3to1(int type)
{
//  xxxxxxxxxx    can be used to close the gap of 3*1 meshes
//  x  |  |  x
//  x--+--+--x
//  x  |  |  x
//  xxxxxxxxxx

	if(!this->ref_i3to1buf.isvalid())
		init3to1Index();// just only one needed, all three use same index

   	if(!this->c_3to1vb.isvalid())
	{
		char iname[256];
		sprintf(iname,"nWaterNode_%d_%d",N_CHOPPY_3TO1, SampleSize);
		//first try to find it then create a new !
		this->c_3to1vb = this->refGfx->FindVertexBuffer(iname);
		if(this->c_3to1vb.isvalid())
		{
			((nVertexBuffer *)(this->c_3to1vb.get()))->AddRef();
		}
		else
		{																				//8 * n-1 *4		
			this->c_3to1vb = this->refGfx->NewVertexBuffer(iname,N_VBTYPE_STATIC,VertType,(32*(SampleSize-1)));
		}
	}

	const nComplex * xmap = getXMap();
	const nComplex * ymap = getCMap();
	const nComplex * zmap = getZMap();

	this->c_3to1vb->LockVertices();
	float *coord = this->c_3to1vb->coord_ptr;
	float *norm = this->c_3to1vb->norm_ptr;
    ulong *color = this->c_3to1vb->color_ptr;
    float *uv0 = this->c_3to1vb->uv_ptr[0];
	ulong ccol = n_f2rgba(1.0f,1.0f,1.0f,1.0f);
	
	int x,z,i,g;
	g=-1;
//	static bool walk=true;
	// walk right
	for(i=0,x=0,z=0;i<(SampleSize *3);i++,x++)
	{
		g++;
		coord[0]=(float)x + xmap[x%SampleSize].real() * 0.04f;
		coord[1]=ymap[x%SampleSize].real() * 0.04f;
		coord[2]=(float)z;
//if(walk)
//n_printf("g: %d\tx: %f\t z:%f\n",g,((float)x + xmap[x%SampleSize].real() * 0.04f),(float)z);
		norm[0] = 0.0f;
		norm[1] = ymap[x%SampleSize].real() * ymap[x%SampleSize].real() * 0.0016f;
		norm[2] = 0.0f;

		color = &ccol;// colors are all the same this could be chnaged in future

		uv0[0] = (float)(x%SampleSize)/SampleSize;
//if(walk)
//n_printf("t: u:%f\tv:%f\n",(float)(x%SampleSize)/SampleSize,0.0f);
		uv0[1] = 0.0f;

		uv0 += this->c_3to1vb->stride4;
		color += this->c_3to1vb->stride4;
		norm += this->c_3to1vb->stride4;
		coord += this->c_3to1vb->stride4;
	}
	//walk down
	for(i=0;i<(SampleSize *3);i++,z++)
	{
		g++;
		coord[0]=(float)x;
		coord[1]=ymap[(z%SampleSize)*SampleSize].real() * 0.04f;
		coord[2]=(float)z + zmap[(z%SampleSize)*SampleSize].real() * 0.04f;
//if(walk)
//n_printf("g: %d\tx: %f\t z:%f\n",g,(float)x,(float)z + zmap[(z%SampleSize)*SampleSize].real() * 0.04f);		
		norm[0] = 0.0f;
		norm[1] = ymap[(z%SampleSize)*SampleSize].real() * ymap[(z%SampleSize)*SampleSize].real() * 0.0016f;
		norm[2] = 0.0f;

		color = &ccol;// colors are all the same this could be chnaged in future

		uv0[0] = 1.0f;
		uv0[1] = (float)(z%SampleSize)/SampleSize;
//if(walk)
//n_printf("t: u:%f\tv:%f\n",1.0f, (float)(z%SampleSize)/SampleSize);

		uv0 += this->c_3to1vb->stride4;
		color += this->c_3to1vb->stride4;
		norm += this->c_3to1vb->stride4;
		coord += this->c_3to1vb->stride4;
	}
	//walks left on bottom
	for(i=0;i<(SampleSize *3);i++,x--)
	{
		g++;
		coord[0]=(float)x + xmap[x%SampleSize].real() * 0.04f;
		coord[1]=ymap[x%SampleSize].real() * 0.04f;
		coord[2]=(float)z;
//if(walk)
//n_printf("g: %d\tx: %f\t z:%f\n",g,(float)x + xmap[x%SampleSize].real() * 0.04f,(float)z);		
		norm[0] = 0.0f;
		norm[1] = ymap[x%SampleSize].real() * ymap[x%SampleSize].real() * 0.0016f;
		norm[2] = 0.0f;

		color = &ccol;// colors are all the same this could be chnaged in future

		uv0[0] = (float)(x%SampleSize)/SampleSize;
		uv0[1] = 1.0f;
//if(walk)
//n_printf("t: u:%f\tv:%f\n",(float)(x%SampleSize)/SampleSize,0.0f);

		uv0 += this->c_3to1vb->stride4;
		color += this->c_3to1vb->stride4;
		norm += this->c_3to1vb->stride4;
		coord += this->c_3to1vb->stride4;
	}
	//walk up to start
	for(i=0;i<(SampleSize *3);i++,z--)
	{
		g++;
		coord[0]=(float)x;
		coord[1]=ymap[(z%SampleSize)*SampleSize].real() * 0.04f;
		coord[2]=(float)z + zmap[(z%SampleSize)*SampleSize].real() * 0.04f;
//if(walk)
//n_printf("g: %d\tx: %f\t z:%f\n",g,(float)x,(float)z + zmap[(z%SampleSize)*SampleSize].real() * 0.04f);		
		norm[0] = 0.0f;
		norm[1] = ymap[(z%SampleSize)*SampleSize].real() * ymap[(z%SampleSize)*SampleSize].real() * 0.0016f;
		norm[2] = 0.0f;

		color = &ccol;// colors are all the same this could be chnaged in future

		uv0[0] = 0.0f;
		uv0[1] = (float)(z%SampleSize)/SampleSize;
//if(walk)
//n_printf("t: u:%f\tv:%f\n",0.0f,(float)(z%SampleSize)/SampleSize);

		uv0 += this->c_3to1vb->stride4;
		color += this->c_3to1vb->stride4;
		norm += this->c_3to1vb->stride4;
		coord += this->c_3to1vb->stride4;
	}
	//now the 4 sides of the outside mesh
	for(i=0,x=0,z=0;i<SampleSize;i++,x++)
	{
		g++;
		coord[0]=((float)x + xmap[x%SampleSize].real() * 0.04f)*3.0f;
		coord[1]=ymap[x%SampleSize].real() * 0.04f;
		coord[2]=(float)z;
//if(walk)
//n_printf("g: %d\tx: %f\t z:%f\n",g,((float)x + xmap[x%SampleSize].real() * 0.04f)*3.0f,(float)z);		
		norm[0] = 0.0f;
		norm[1] = ymap[x%SampleSize].real() * ymap[x%SampleSize].real() * 0.0016f;
		norm[2] = 0.0f;

		color = &ccol;// colors are all the same this could be chnaged in future
		//UV must be scaled by shadernode
		uv0[0] = (float)(x%SampleSize)/SampleSize;
		uv0[1] = 1.0f;
//if(walk)
//n_printf("t: u:%f\tv:%f\n",(float)(x%SampleSize)/SampleSize,1.0f);
	
		uv0 += this->c_3to1vb->stride4;
		color += this->c_3to1vb->stride4;
		norm += this->c_3to1vb->stride4;
		coord += this->c_3to1vb->stride4;
	}
	//walk down
	for(i=0;i<SampleSize;i++,z++)
	{
		g++;
		coord[0]=(float)x*3.0f;
		coord[1]=ymap[(z%SampleSize)*SampleSize].real() * 0.04f;
		coord[2]=((float)z + zmap[(z%SampleSize)*SampleSize].real() * 0.04f)*3.0f;
//if(walk)
//n_printf("g: %d\tx: %f\t z:%f\n",g,(float)x*3.0f,((float)z + zmap[(z%SampleSize)*SampleSize].real() * 0.04f)*3.0f);		
		norm[0] = 0.0f;
		norm[1] = ymap[(z%SampleSize)*SampleSize].real() * ymap[(z%SampleSize)*SampleSize].real() * 0.0016f;
		norm[2] = 0.0f;

		color = &ccol;// colors are all the same this could be chnaged in future

		uv0[0] = 0.0f;
		uv0[1] = (float)(z%SampleSize)/SampleSize;
//if(walk)
//n_printf("t: u:%f\tv:%f\n",0.0f,(float)(z%SampleSize)/SampleSize);
	
		uv0 += this->c_3to1vb->stride4;
		color += this->c_3to1vb->stride4;
		norm += this->c_3to1vb->stride4;
		coord += this->c_3to1vb->stride4;
	}
	//walks left on bottom
	for(i=0;i<SampleSize;i++,x--)
	{
		g++;
		coord[0]=((float)x + xmap[x%SampleSize].real() * 0.04f)*3.0f;
		coord[1]=ymap[x%SampleSize].real() * 0.04f;
		coord[2]=(float)z*3.0f;
//if(walk)
//n_printf("g: %d\tx: %f\t z:%f\n",g,((float)x + xmap[x%SampleSize].real() * 0.04f)*3.0f,(float)z*3.0f);		
		norm[0] = 0.0f;
		norm[1] = ymap[x%SampleSize].real() * ymap[x%SampleSize].real() * 0.0016f;
		norm[2] = 0.0f;

		color = &ccol;// colors are all the same this could be chnaged in future

		uv0[0] = (float)(x%SampleSize)/SampleSize;
		uv0[1] = 0.0f;
//if(walk)
//n_printf("t: u:%f\tv:%f\n",(float)(x%SampleSize)/SampleSize,0.0f);
	
		uv0 += this->c_3to1vb->stride4;
		color += this->c_3to1vb->stride4;
		norm += this->c_3to1vb->stride4;
		coord += this->c_3to1vb->stride4;
	}
	//walk up to start
	for(i=0;i<SampleSize;i++,z--)
	{
		g++;
		coord[0]=(float)x*3.0f;
		coord[1]=ymap[(z%SampleSize)*SampleSize].real() * 0.04f;
		coord[2]=((float)z + zmap[(z%SampleSize)*SampleSize].real() * 0.04f)*3.0f;
//if(walk)
//n_printf("g: %d\tx: %f\t z:%f\n",g,(float)x*3.0f,((float)z + zmap[(z%SampleSize)*SampleSize].real() * 0.04f)*3.0f);		
		norm[0] = 0.0f;
		norm[1] = ymap[(z%SampleSize)*SampleSize].real() * ymap[(z%SampleSize)*SampleSize].real() * 0.0016f;
		norm[2] = 0.0f;

		color = &ccol;// colors are all the same this could be chnaged in future

		uv0[0] = 1.0f;
		uv0[1] = (float)(z%SampleSize)/SampleSize;
//if(walk)
//n_printf("t: u:%f\tv:%f\n",1.0f,(float)(z%SampleSize)/SampleSize);

		uv0 += this->c_3to1vb->stride4;
		color += this->c_3to1vb->stride4;
		norm += this->c_3to1vb->stride4;
		coord += this->c_3to1vb->stride4;
	}
//walk=false;

	this->c_3to1vb->UnlockVertices();
}

// height query based on the grid will NOT be the position at the 
float nWaterServer::GetHeight(int x, int z, int algo)
{
	switch(algo)
	{
	case N_STATISTIC: 
	case N_CHOPPY: return (this->m_Choppy->getMap())[x+(z*SampleSize)].real()*0.04f;
	case N_GERSTNER: return (this->m_Gerstner->getVertices())[x+(z*SampleSize)].y *0.04f;
	default: return 0.0f;
	}
}
// height query based on the desired position above the grid 
float nWaterServer::GetHeight(float x, float z, int algo)
{
	switch(algo)
	{
	case N_STATISTIC: 
		return GetSH(x,z);
//	case N_CHOPPY: return (this->m_Choppy->getMap())[x+(z*SampleSize)].real()*0.04f;
//	case N_GERSTNER: return (this->m_Gerstner->getVertices())[x+(z*SampleSize)].y *0.04f;
	default: return 0.0f;
	}
}

//statistic height no deviation in x and z simples case of all
float nWaterServer::GetSH(float x, float z)
{
	int _x=x;
	int _z=z;
n_printf("_x: %d _z: %d",_x, _z);
	if(z<=0.0f)
	{
		//lower half of quad
		float ya = (this->m_Choppy->getMap())[_x+(_z*SampleSize)].real();
		float yb = (this->m_Choppy->getMap())[_x+1+(_z*SampleSize)].real();
		float yc = (this->m_Choppy->getMap())[_x+1+((_z+1)*SampleSize)].real();
n_printf(" %f =(%f-%d)*(%f-%f) + (%f-%d)*(%f-%f) *0.04f \n",(( (z-_z)*(yb-ya) + (x-_x)*(yc-yb) ) * 0.04f),z,_z,yb,ya,x,_x,yc,yb );
		return ( (z-_z)*(yb-ya) + (x-_x)*(yc-yb) ) * 0.04f;
	}
	else if(x/z <= 1.0f)
	{
		//lower half of quad
		float ya = (this->m_Choppy->getMap())[_x+(_z*SampleSize)].real();
		float yb = (this->m_Choppy->getMap())[_x+1+(_z*SampleSize)].real();
		float yc = (this->m_Choppy->getMap())[_x+1+((_z+1)*SampleSize)].real();
n_printf(" %f =(%f-%d)*(%f-%f) + (%f-%d)*(%f-%f) *0.04f \n",(( (z-_z)*(yb-ya) + (x-_x)*(yc-yb) ) * 0.04f),z,_z,yb,ya,x,_x,yc,yb );
		return ( (z-_z)*(yb-ya) + (x-_x)*(yc-yb) ) * 0.04f;

	}
	else
	{
		//upper half of quad
		float ya = (this->m_Choppy->getMap())[_x+(_z*SampleSize)].real();
		float yd = (this->m_Choppy->getMap())[_x+((_z+1)*SampleSize)].real();
		float yc = (this->m_Choppy->getMap())[_x+1+((_z+1)*SampleSize)].real();
n_printf(" %f =(%f-%d)*(%f-%f) + (%f-%d)*(%f-%f) *0.04f \n",(( (z-_z)*(yd-ya) + (x-_x)*(yc-yd) ) * 0.04f),z,_z,yd,ya,x,_x,yc,yd );
		return ( (z-_z)*(yd-ya) + (x-_x)*(yc-yd) ) * 0.04f;
	}
}
